@@ -1,5 +1,4 @@
 ////////// MemviewSliceStruct.proto //////////
-//@proto_block: utility_code_proto_before_types
 
 /* memoryview slice struct */
 struct {{memview_struct_name}};
@@ -12,12 +11,8 @@ typedef struct {
   Py_ssize_t suboffsets[{{max_dims}}];
 } {{memviewslice_name}};
 
-// used for "len(memviewslice)"
-#define __Pyx_MemoryView_Len(m)  (m.shape[0])
-
 
 /////////// Atomics.proto /////////////
-//@proto_block: utility_code_proto_before_types
 
 #include <pythread.h>
 
@@ -82,7 +77,7 @@ typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
 
 /////////////// ObjectToMemviewSlice.proto ///////////////
 
-static CYTHON_INLINE {{memviewslice_name}} {{funcname}}(PyObject *, int writable_flag);
+static CYTHON_INLINE {{memviewslice_name}} {{funcname}}(PyObject *);
 
 
 ////////// MemviewSliceInit.proto //////////
@@ -127,7 +122,7 @@ static CYTHON_INLINE char *__pyx_memviewslice_index_full(
 /////////////// ObjectToMemviewSlice ///////////////
 //@requires: MemviewSliceValidateAndInit
 
-static CYTHON_INLINE {{memviewslice_name}} {{funcname}}(PyObject *obj, int writable_flag) {
+static CYTHON_INLINE {{memviewslice_name}} {{funcname}}(PyObject *obj) {
     {{memviewslice_name}} result = {{memslice_init}};
     __Pyx_BufFmt_StackElem stack[{{struct_nesting_depth}}];
     int axes_specs[] = { {{axes_specs}} };
@@ -140,7 +135,7 @@ static CYTHON_INLINE {{memviewslice_name}} {{funcname}}(PyObject *obj, int writa
     }
 
     retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, {{c_or_f_flag}},
-                                                 {{buf_flag}} | writable_flag, {{ndim}},
+                                                 {{buf_flag}}, {{ndim}},
                                                  &{{dtype_typeinfo}}, stack,
                                                  &result, obj);
 
@@ -169,8 +164,6 @@ static int __Pyx_ValidateAndInit_memviewslice(
 
 /////////////// MemviewSliceValidateAndInit ///////////////
 //@requires: Buffer.c::TypeInfoCompare
-//@requires: Buffer.c::BufferFormatStructs
-//@requires: Buffer.c::BufferFormatCheck
 
 static int
 __pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
@@ -244,7 +237,7 @@ __pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec
     }
 
     if (spec & __Pyx_MEMVIEW_PTR) {
-        if (!buf->suboffsets || (buf->suboffsets[dim] < 0)) {
+        if (!buf->suboffsets || (buf->suboffsets && buf->suboffsets[dim] < 0)) {
             PyErr_Format(PyExc_ValueError,
                          "Buffer is not indirectly accessible "
                          "in dimension %d.", dim);
@@ -394,7 +387,11 @@ __Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
     Py_buffer *buf = &memview->view;
     __Pyx_RefNannySetupContext("init_memviewslice", 0);
 
-    if (memviewslice->memview || memviewslice->data) {
+    if (!buf) {
+        PyErr_SetString(PyExc_ValueError,
+            "buf is NULL.");
+        goto fail;
+    } else if (memviewslice->memview || memviewslice->data) {
         PyErr_SetString(PyExc_ValueError,
             "memviewslice is already initialized!");
         goto fail;
@@ -440,12 +437,8 @@ no_fail:
     return retval;
 }
 
-#ifndef Py_NO_RETURN
-// available since Py3.3
-#define Py_NO_RETURN
-#endif
 
-static void __pyx_fatalerror(const char *fmt, ...) Py_NO_RETURN {
+static CYTHON_INLINE void __pyx_fatalerror(const char *fmt, ...) {
     va_list vargs;
     char msg[200];
 
@@ -454,10 +447,11 @@ static void __pyx_fatalerror(const char *fmt, ...) Py_NO_RETURN {
 #else
     va_start(vargs);
 #endif
-    vsnprintf(msg, 200, fmt, vargs);
-    va_end(vargs);
 
+    vsnprintf(msg, 200, fmt, vargs);
     Py_FatalError(msg);
+
+    va_end(vargs);
 }
 
 static CYTHON_INLINE int
@@ -695,21 +689,29 @@ __pyx_slices_overlap({{memviewslice_name}} *slice1,
 }
 
 
-////////// MemviewSliceCheckContig.proto //////////
+////////// MemviewSliceIsCContig.proto //////////
 
-#define __pyx_memviewslice_is_contig_{{contig_type}}{{ndim}}(slice) \
-    __pyx_memviewslice_is_contig(slice, '{{contig_type}}', {{ndim}})
+#define __pyx_memviewslice_is_c_contig{{ndim}}(slice) \
+        __pyx_memviewslice_is_contig(slice, 'C', {{ndim}})
+
+
+////////// MemviewSliceIsFContig.proto //////////
+
+#define __pyx_memviewslice_is_f_contig{{ndim}}(slice) \
+        __pyx_memviewslice_is_contig(slice, 'F', {{ndim}})
 
 
 ////////// MemviewSliceIsContig.proto //////////
 
-static int __pyx_memviewslice_is_contig(const {{memviewslice_name}} mvs, char order, int ndim);/*proto*/
+static int __pyx_memviewslice_is_contig(const {{memviewslice_name}} mvs,
+                                        char order, int ndim);
 
 
 ////////// MemviewSliceIsContig //////////
 
 static int
-__pyx_memviewslice_is_contig(const {{memviewslice_name}} mvs, char order, int ndim)
+__pyx_memviewslice_is_contig(const {{memviewslice_name}} mvs,
+                             char order, int ndim)
 {
     int i, index, step, start;
     Py_ssize_t itemsize = mvs.memview->view.itemsize;
@@ -848,40 +850,28 @@ if (unlikely(__pyx_memoryview_slice_memviewslice(
 
 {
     Py_ssize_t __pyx_tmp_idx = {{idx}};
-
-    {{if wraparound or boundscheck}}
-        Py_ssize_t __pyx_tmp_shape = {{src}}.shape[{{dim}}];
-    {{endif}}
-
+    Py_ssize_t __pyx_tmp_shape = {{src}}.shape[{{dim}}];
     Py_ssize_t __pyx_tmp_stride = {{src}}.strides[{{dim}}];
-    {{if wraparound}}
-        if (__pyx_tmp_idx < 0)
-            __pyx_tmp_idx += __pyx_tmp_shape;
-    {{endif}}
+    if ({{wraparound}} && (__pyx_tmp_idx < 0))
+        __pyx_tmp_idx += __pyx_tmp_shape;
 
-    {{if boundscheck}}
-        if (!__Pyx_is_valid_index(__pyx_tmp_idx, __pyx_tmp_shape)) {
-            {{if not have_gil}}
-                #ifdef WITH_THREAD
-                PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
-                #endif
-            {{endif}}
+    if ({{boundscheck}} && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
+        {{if not have_gil}}
+            #ifdef WITH_THREAD
+            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
+            #endif
+        {{endif}}
 
-            PyErr_SetString(PyExc_IndexError,
-                            "Index out of bounds (axis {{dim}})");
+        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis {{dim}})");
 
-            {{if not have_gil}}
-                #ifdef WITH_THREAD
-                PyGILState_Release(__pyx_gilstate_save);
-                #endif
-            {{endif}}
+        {{if not have_gil}}
+            #ifdef WITH_THREAD
+            PyGILState_Release(__pyx_gilstate_save);
+            #endif
+        {{endif}}
 
-            {{error_goto}}
-        }
-    {{else}}
-        // make sure label is not un-used
-        if ((0)) {{error_goto}}
-    {{endif}}
+        {{error_goto}}
+    }
 
     {{if all_dimensions_direct}}
         {{dst}}.data += __pyx_tmp_idx * __pyx_tmp_stride;
